library("gWidgets")
library("gWidgetstcltk")


library("utils")       # to use csv read function
library("foreign")     # to use spss read function
library("pastecs")     # to use descriptive statistics function
library("mlogit")
library("graphics")
library("VGAM")
library("ZeligChoice") # This package is for actual mlogit function
library("aod")
library("plotrix")
library("MASS")        # For ordered Probit
library("ordinal")     # Also for ordered probit
library("lmtest")      # for likelihood ratio test
library("miscTools")
library("maxLik")


rm(list = ls());
win <- gwindow("Vehicle Fleet Composition");

group <- ggroup(horizontal=FALSE,container = win, spacing=0);
nb <- gnotebook(container = group, tab.pos=3);


VFC_Folder <- ggroup(horizontal=FALSE, container = nb, label="0 Setting");
tbl <- glayout(container=VFC_Folder, expand=TRUE);
curfolder_lab <- glabel("Project Folder", container = tbl);
curfolder_var <- gedit("C:/workspace/VFC_Package_Beta/Model_Application/", container = tbl);
halton_lab <- glabel("Halton File", container = tbl);
halton_var <- gedit("C:/workspace/VFC_Package_Beta/Halton_R/haltbrat_vfc101.csv", container = tbl);
input_lab <- glabel("Input Data", container = tbl);
input_var <- gedit("C:/workspace/VFC_Package_Beta/Model_Application/az_hhld_vfc_cleaned_final.csv", container = tbl);
output_lab <- glabel("Output Data", container = tbl);
output_var <- gedit("C:/workspace/VFC_Package_Beta/Model_Application/VFC_output.csv", container = tbl);
seed_lab <- glabel("Seed", container = tbl);
seed_var <- gedit("1", container = tbl);
size(curfolder_var) <- c(400, 15);
size(halton_var) <- c(400, 15);
size(input_var) <- c(400, 15);
size(output_var) <- c(400, 15);

tbl[1,1] <- curfolder_lab
tbl[1,2] <- curfolder_var
tbl[2,1] <- halton_lab
tbl[2,2] <- halton_var
tbl[3,1] <- input_lab
tbl[3,2] <- input_var
tbl[4,1] <- output_lab
tbl[4,2] <- output_var
tbl[5,1] <- seed_lab
tbl[5,2] <- seed_var


set.seed(as.numeric(svalue(seed_var)));

model1 <- ggroup(horizontal=FALSE, container = nb, label="1 Mileage Prediction");


coef_names_mile <- c('const','DRVRCNT','NUMADLT','RUR','INCOME1','INCOME5','NUMCHILD','WORK0','WORK2','HSIZE4','SF_Own','Retired','Prop_HH_INC5', 'Prop_SFHH', 'AUTO10_Q1');
coef_mile <- c(13.03, 2.01, 0.374, 0.867, -1.05, 1.325, 0.517, -1.513, 0.829, -0.669, 0.570, -0.878, 1.362, 0.687, -0.293);


tbl <- glayout(container=model1, spacing=3);

glabels_m1 <- c();
gedits_m1 <- c();
for (i in 1:(length(coef_names_mile))) {
  
  var_name <- coef_names_mile[[i]]
  var_value <- coef_mile[[i]]
  
  tmp_label <- glabel(var_name, container = tbl)
  tmp_var <- gedit(var_value, container = tbl)
  
  
  tbl[i,1] <- tmp_label
  tbl[i,2] <- tmp_var
  
  glabels_m1 <- c(glabels_m1, c(tmp_label))
  gedits_m1 <- c(gedits_m1, c(tmp_var))
  
}


#####################################################################################################################################################
# 2
# MNL model for number of alternatives
# Calibrated coefficients

model2 <- ggroup(container = nb, label="2 Number of Alternatives", horizontal=FALSE);

coefs_name_0veh <- c('const', 'INCOME1', 'INCOME2', 'OWN', 'HSIZE1', 'WORK0', 'POPDEN');
coefs_0veh <- c(1.73, 2.37, 0.82, -1.65, 2.12, 1.21, 0.00011);
coefs_name_1veh <- c('const', 'INCOME1', 'INCOME2', 'HSIZE1', 'Prop_MFHH', 'WORK2');
coefs_1veh <- c(4.27, 1.43, 0.96, 2.22, 0.4, -0.86);
coefs_name_2veh <- c('const', 'lif_cyc_1', 'INCOME3', 'WORK2', 'INC1');
coefs_2veh <- c(5.31, 0.42, -0.21, -0.34, -0.00034);
coefs_name_3veh <- c('const', 'OWN', 'NUMADLT', 'WORK3', 'POPDEN', 'child_pres', 'lif_cyc_2');
coefs_3veh <- c(1.69, 1.0, 0.48, 1.02, -0.000088, 0.38, 0.5);
coefs_name_4veh <- c('const', 'INCOME5', 'lif_cyc_2', 'HSIZE4', 'WORK2');
coefs_4veh <- c(1.18, 0.72, 1.63, 1.51, -0.7);
coefs_name_5veh <- c('const');
coefs_5veh <- c(0.0);

coef_names_m2 <- list(coefs_name_0veh, coefs_name_1veh, coefs_name_2veh , coefs_name_3veh, coefs_name_4veh, coefs_name_5veh);
coef_values_m2 <- list(coefs_0veh, coefs_1veh, coefs_2veh, coefs_3veh, coefs_4veh, coefs_5veh);

glabel_list_m2 <- list();
gedit_list_m2 <- list();

vehtype_names <- c("0 Vehicles","1 Vehicle Type","2 Vehicle Types","3 Vehicle Types","4 Vehicle Types","5+ Vehicle Types");

tmp_group_list <- list();
for(i in 1:(length(coef_names_m2)%/%5+1)) {
  tmp_group_list[[i]] <- ggroup(container = model2, anchor=c(-1,1));
}

tbl_m2 <- glayout(container=model2, expand=TRUE);


for(i in 1:(length(coef_names_m2))) {
  
  names <- coef_names_m2[[i]];
  values <- coef_values_m2[[i]];

  
  ind <- (i-1)%/%5 + 1;
  tmp_group <- ggroup(container = tmp_group_list[[ind]], anchor=c(-1,1), expand=TRUE);
  tmp_frame <- gframe(vehtype_names[i], container=tmp_group, horizontal=FALSE, anchor=c(-1,1), expand=TRUE);
  
  
  
  tbl <- glayout(container=tmp_frame, spacing=1, expand=TRUE, anchor=c(-1,1));
  
  
  glabels_m2 <- c();
  gedits_m2 <- c();
  for (j in 1:(length(values))) {
    
    var_name <- names[[j]];
    var_value <- values[[j]];
    
    tmp_label <- glabel(var_name, container = tbl);
    tmp_var <- gedit(var_value, container = tbl);
    size(tmp_var) <- c(100, 20);
    
    tbl[j,1] <- tmp_label;
    tbl[j,2] <- tmp_var;
    
    glabels_m2 <- c(glabels_m2, c(tmp_label))
    gedits_m2 <- c(gedits_m2, c(tmp_var))
  }
  
  glabel_list_m2[[i]] <- glabels_m2;
  gedit_list_m2[[i]] <- gedits_m2;
}



model3 <- ggroup(container = nb, label="3 Vehicle Fleet Mix (MDCEV)", horizontal=FALSE);

mdcev_group <- ggroup(horizontal=FALSE,container = model3, spacing=0);
mdcev_nb <- gnotebook(container = mdcev_group, tab.pos=3);


mdcev_set_lab <- list();
mdcev_set_var <- list();

model3_set <- ggroup(container = mdcev_nb, label="Setting for MDCEV", horizontal=FALSE);
tbl <- glayout(container=model3_set, expand=TRUE);
mdcev_set_lab[[1]] <- glabel("UNO - One (Column Index)", container = tbl);
mdcev_set_var[[1]] <- gedit("5", container = tbl);
mdcev_set_lab[[2]] <- glabel("SERO - Zero (Column Index)", container = tbl);
mdcev_set_var[[2]] <- gedit("6", container = tbl);
mdcev_set_lab[[3]] <- glabel("Case ID (Column Index)", container = tbl);
mdcev_set_var[[3]] <- gedit("7", container = tbl);
mdcev_set_lab[[4]] <- glabel("Number of Replications", container = tbl);
mdcev_set_var[[4]] <- gedit("25", container = tbl);
mdcev_set_lab[[5]] <- glabel("Number of Outside Goods", container = tbl);
mdcev_set_var[[5]] <- gedit("1", container = tbl);
mdcev_set_lab[[6]] <- glabel("Configuration (1,4,5,6, or 7)", container = tbl);
mdcev_set_var[[6]] <- gedit("4", container = tbl);

tbl[1,1] <- mdcev_set_lab[[1]]
tbl[1,2] <- mdcev_set_var[[1]]
tbl[2,1] <- mdcev_set_lab[[2]]
tbl[2,2] <- mdcev_set_var[[2]]
tbl[3,1] <- mdcev_set_lab[[3]]
tbl[3,2] <- mdcev_set_var[[3]]
tbl[4,1] <- mdcev_set_lab[[4]]
tbl[4,2] <- mdcev_set_var[[4]]
tbl[5,1] <- mdcev_set_lab[[5]]
tbl[5,2] <- mdcev_set_var[[5]]
tbl[6,1] <- mdcev_set_lab[[6]]
tbl[6,2] <- mdcev_set_var[[6]]



#### Baseline utility equation
alt_names <- c("Car 0-5","Car 6-11","Car 12 or older","Van 0-5","Van 6-11","Van 12 or older","SUV 0-5","SUV 6-11","SUV 12 or older",
               "Pick-up 0-5","Pick-up 6-11","Pick-up 12 or older","Motor-bike")
# Car 0 - 5 years old
ivmt2 <- c("uno", "INCOME1", "INCOME4", "NUMCHILD", "WORK3", "Prop_HH_INC5", "RATE_AUTO10");
mcoef2 <- c(-4.542, -0.4, 0.164, -0.189, 0.173, -1.013, -13.833);

# Car 6 - 11 years old
ivmt3 <- c("uno", "WORK2", "INCOME2");
mcoef3 <- c(-5.057, -0.157, 0.132);

# Car 12 years or older
ivmt4 <- c("uno", "INCOME1", "Retired", "Prop_HH_INC1");
mcoef4 <- c(-5.846, 0.574, 0.202, 0.574);

# Van 0 - 5 years old
ivmt5 <- c("uno", "NUMCHILD", "WORK2", "TAZ_den_Q1", "RATE_AUTO30");
mcoef5 <- c(-7.479, 0.379, -0.388, -0.307, -1.717);

# Van 6-11 years old
ivmt6 <- c("uno", "NUMCHILD", "TAZ_den_Q1", "INCOME2");
mcoef6 <- c(-7.579, 0.331, -0.258, 0.269);

# Van 12 years or older
ivmt7 <- c("uno", "HHSIZE", "TAZ_den_Q1", "INCOME1");
mcoef7 <- c(-8.897, 0.138, 0.676, 0.659);

# SUV 0 - 5 years old
ivmt8 <- c("uno", "INCOME1", "WORK2", "Retired", "Prop_HH_INC1", "RATE_AUTO10", "TAZ_den_Q3");
mcoef8 <- c(-5.815, -1.014, 0.169, -0.167, -1.475, -17.83, 0.267);

# SUV 6-11 years old
ivmt9 <- c("uno", "INCOME3", "HSIZE4", "SF_Own", "AUTO30_Q1");
mcoef9 <- c(-7.371, 0.257, 0.333, 0.86, -0.301);

# SUV 12 years or older
ivmt10 <- c("uno", "INCOME4", "child_pres", "SF", "TAZ_den_Q2");
mcoef10 <- c(-7.030, 0.36, 0.313, -0.736, -0.361);

# Pick-up 0-5 years old
ivmt11 <- c("uno", "INCOME5", "HSIZE1", "RUR", "Prop_SFHH");
mcoef11 <- c(-7.616, 0.244, -0.978, 0.239, 0.74);

# Pick-up 6-11 years old
ivmt12 <- c("uno", "RUR", "Retired", "INCOME4", "AUTO10_Q1");
mcoef12 <- c(-6.583, 0.153, -0.353, 0.164, -0.229);

# Pick-up 12 years or older
ivmt13 <- c("uno", "Prop_HH_INC1", "INCOME2", "child_pres", "AUTO10_Q1");
mcoef13 <- c(-7.177, 1.283, 0.345, -0.25, -0.335);

# Motorbike
ivmt14 <- c("uno", "RUR", "SF_Own", "HSIZE1");
mcoef14 <- c(-8.190, 0.705, 0.752, -0.567);


ivmts <- list(ivmt2,ivmt3,ivmt4,ivmt5,ivmt6,ivmt7,ivmt8,ivmt9,ivmt10,ivmt11,ivmt12,ivmt13,ivmt14);
ivmts_coef <- c(mcoef2,mcoef3,mcoef4,mcoef5,mcoef6,mcoef7,mcoef8,mcoef9,mcoef10,mcoef11,mcoef12,mcoef13,mcoef14);


coef_names_m3 <- list(ivmt2,ivmt3,ivmt4,ivmt5,ivmt6,ivmt7,ivmt8,ivmt9,ivmt10,ivmt11,ivmt12,ivmt13,ivmt14);
coef_values_m3 <- list(mcoef2,mcoef3,mcoef4,mcoef5,mcoef6,mcoef7,mcoef8,mcoef9,mcoef10,mcoef11,mcoef12,mcoef13,mcoef14);


glabel_list_m3 <- list();
gedit_list_m3 <- list();



model3_1 <- ggroup(container = mdcev_nb, label="Gamma Parameter", horizontal=FALSE, spacing=0);

tmp_group_list <- list();
for(i in 1:(length(coef_names_m3)%/%5+1)) {
  tmp_group_list[[i]] <- ggroup(container = model3_1, anchor=c(-1,1), spacing=0);
}


for(i in 1:(length(coef_names_m3))) {
  
  names <- coef_names_m3[[i]];
  values <- coef_values_m3[[i]];

  ind <- 1+(i-1)%/%5;
  tmp_group <- ggroup(container = tmp_group_list[[ind]], anchor=c(-1,1), expand=TRUE);
  tmp_frame <- gframe(alt_names[i], container=tmp_group, horizontal=FALSE, anchor=c(-1,1), expand=TRUE);
  
  tbl <- glayout(container=tmp_frame, spacing=0, expand=TRUE, anchor=c(-1,1));
  
  
  glabels_m3 <- c();
  gedits_m3 <- c();
  for (j in 1:(length(values))) {
    
    var_name <- names[[j]];
    var_value <- values[[j]];
    
    tmp_label <- glabel(var_name, container = tbl, anchor=c(-1,1));
    tmp_var <- gedit(var_value, container = tbl, anchor=c(1,1));
    size(tmp_var) <- c(90, 20); 
    
    tbl[j,1] <- tmp_label;
    tbl[j,2] <- tmp_var;
    
    glabels_m3 <- c(glabels_m3, c(tmp_label))
    gedits_m3 <- c(gedits_m3, c(tmp_var))
  }
  
  glabel_list_m3[[i]] <- glabels_m3;
  gedit_list_m3[[i]] <- gedits_m3;
}


#### Translation parameters
# Car 0 - 5 years old
ivgt2 <- c("uno",  "INCOME1", "INCOME2", "INCOME3");
gcoef2 <- c(8.072, 0.3, -0.35, -0.15);

# Car 6 - 11 years old
ivgt3 <- c("uno", "INCOME1");
gcoef3 <- c(7.732, 0.35);

# Car 12 years or older
ivgt4 <- c("uno",  "INCOME1");
gcoef4 <- c(7.806, 0.2);

# Van 0 - 5 years old
ivgt5 <- c("uno",	"INCOME1", "INCOME2");
gcoef5 <- c(10.889, 1.55, -0.5);

# Van 6-11 years old
ivgt6 <- c("uno",	"INCOME2", "INCOME3");
gcoef6 <- c(9.810, 0.5, 1.5);

# Van 12 years or older
ivgt7 <- c("uno",	"INCOME1", "INCOME2", "INCOME3");
gcoef7 <- c(10.949, -0.2, -1.0, 3.0);

# SUV 0 - 5 years old
ivgt8 <- c("uno",	"INCOME1", "INCOME2");
gcoef8 <- c(9.233, 0.35, -0.65);

# SUV 6-11 years old
ivgt9 <- c("uno");
gcoef9 <- c(9.024);

# SUV 12 years or older
ivgt10 <- c("uno");
gcoef10 <- c(9.156);

# Pick-up 0-5 years old
ivgt11 <- c("uno");
gcoef11 <- c(10.134);

# Pick-up 6-11 years old
ivgt12 <- c("uno");
gcoef12 <- c(9.001);

# Pick-up 12 years or older
ivgt13 <- c("uno");
gcoef13 <- c(9.164);

# Motorbike
ivgt14 <- c("uno", "INCOME1", "INCOME2", "INCOME3");
gcoef14 <- c(7.507, 1.0, -1.0, 0.5);


coef_names_m32 <- list(ivgt2,ivgt3,ivgt4,ivgt5,ivgt6,ivgt7,ivgt8,ivgt9,ivgt10,ivgt11,ivgt12,ivgt13,ivgt14);
coef_values_m32 <- list(gcoef2,gcoef3,gcoef4,gcoef5,gcoef6,gcoef7,gcoef8,gcoef9,gcoef10,gcoef11,gcoef12,gcoef13,gcoef14);


glabel_list_m32 <- list();
gedit_list_m32 <- list();

model3_2 <- ggroup(container = mdcev_nb, label="Translation Parameter", horizontal=FALSE)

tmp_group_list <- list();
for(i in 1:(length(coef_names_m32)%/%5+1)) {
  tmp_group_list[[i]] <- ggroup(container = model3_2, anchor=c(-1,1));
}


for(i in 1:(length(coef_names_m32))) {
  
  names <- coef_names_m32[[i]];
  values <- coef_values_m32[[i]];
  
  ind <- 1+(i-1)%/%5;
  tmp_group <- ggroup(container = tmp_group_list[[ind]], anchor=c(-1,1), expand=TRUE);
  tmp_frame <- gframe(alt_names[i], container=tmp_group, horizontal=FALSE, anchor=c(-1,1), expand=TRUE);
  
  tbl <- glayout(container=tmp_frame, spacing=0, expand=TRUE, anchor=c(-1,1));
  
  
  glabels_m32 <- c();
  gedits_m32 <- c();
  for (j in 1:(length(values))) {
    
    var_name <- names[[j]];
    var_value <- values[[j]];
    
    tmp_label <- glabel(var_name, container = tbl, anchor=c(-1,1));
    tmp_var <- gedit(var_value, container = tbl, anchor=c(1,1));
    size(tmp_var) <- c(100, 20);
    
    tbl[j,1] <- tmp_label;
    tbl[j,2] <- tmp_var;
    
    glabels_m32 <- c(glabels_m32, c(tmp_label))
    gedits_m32 <- c(gedits_m32, c(tmp_var))
  }
  
  glabel_list_m32[[i]] <- glabels_m32;
  gedit_list_m32[[i]] <- gedits_m32;
}


model4 <- ggroup(container = nb, label="4 Heuristic Mileage Reallocation")


tmp_group <- ggroup(container = model4, anchor=c(-1,1));
tbl <- glayout(container=tmp_group);
glabel_tol <- glabel("Tolerance", container = tbl);
gedit_tol <- gedit("0.035", container = tbl);

tbl[1,1] <- glabel_tol;
tbl[1,2] <- gedit_tol;

glabel_maxiter <- glabel("Maximum Iteration", container = tbl);
gedit_maxiter <- gedit("5", container = tbl);

tbl[2,1] <- glabel_maxiter;
tbl[2,2] <- gedit_maxiter;



model5 <- ggroup(container = nb, label="5 Number of Body Types",horizontal=FALSE);

coefs_name_0type <- c('const', 'INCOME1', 'INCOME2', 'OWN', 'HSIZE1', 'WORK0', 'POPDEN');
coefs_0type <- c(0.68, 2.093, 0.608, -1.430, 2.109, 1.145, 0.00011);
coefs_name_1type <- c('const', 'INCOME1', 'INCOME2', 'HSIZE1', 'Prop_MFHH', 'child_pres');
coefs_1type <- c(3.672, 1.015, 0.646, 1.995, 0.260, -0.333);
coefs_name_2type <- c('const', 'RUR', 'INCOME4', 'INCOME5', 'HSIZE4','OWN');
coefs_2type <- c(2.956, 0.190, 0.353, 0.230, 0.303, 0.983);
coefs_name_3type <- c('const', 'OWN', 'NUMADLT', 'WORK3', 'POPDEN', 'child_pres');
coefs_3type <- c(0.0, 1.418, 0.548, 0.460, -0.0001, 0.364);
coefs_name_4type <- c('const');
coefs_4type <- c(0.0);

coef_names_m5 <- list(coefs_name_0type, coefs_name_1type, coefs_name_2type , coefs_name_3type, coefs_name_4type);
coef_values_m5 <- list(coefs_0type, coefs_1type, coefs_2type, coefs_3type, coefs_4type);

glabel_list_m5 <- list();
gedit_list_m5 <- list();

bodytype_names <- c("0 Vehicles","1 Body Type","2 Body Types","3 Body Types","4+ Body Types");

tmp_group_list[[1]] <- ggroup(container = model5, anchor=c(-1,1));
tmp_group_list[[2]] <- ggroup(container = model5, anchor=c(-1,1));
k <- 1;
for(i in 1:(length(coef_names_m5))) {
  
  names <- coef_names_m5[[i]];
  values <- coef_values_m5[[i]];
  
  if (i > 4) {k<-2;}
  tmp_group <- ggroup(container = tmp_group_list[[k]], anchor=c(-1,1), expand=TRUE);
  #tmp_label <- paste((i-1),"Vehicle Types",sep=" ");
  tmp_frame <- gframe(bodytype_names[i], container=tmp_group, horizontal=FALSE, expand=TRUE);
  
  tbl <- glayout(container=tmp_frame);
  
  
  glabels_m5 <- c();
  gedits_m5 <- c();
  for (j in 1:(length(values))) {
    
    var_name <- names[[j]];
    var_value <- values[[j]];
    
    tmp_label <- glabel(var_name, container = tbl);
    tmp_var <- gedit(var_value, container = tbl);
    size(tmp_var) <- c(100, 20);
    
    tbl[j,1] <- tmp_label;
    tbl[j,2] <- tmp_var;
    
    glabels_m5 <- c(glabels_m5, c(tmp_label))
    gedits_m5 <- c(gedits_m5, c(tmp_var))
  }
  
  glabel_list_m5[[i]] <- glabels_m5;
  gedit_list_m5[[i]] <- gedits_m5;
}




model6 <- ggroup(container = nb, label="6 Count")

tmp_group <- ggroup(container = model6, anchor=c(-1,1), expand=TRUE);
tmp_frame <- gframe("6.1 Car Count", container=tmp_group, horizontal=FALSE, expand=TRUE);

tbl <- glayout(container=tmp_frame)

coef_names_car <- c('const', 'YCAR0_5', 'YCAR6_11', 'INCOME2', 'INCOME5', 'WORK3', 'NUMADLT', 'HSIZE1', 'HSIZE4', 'OWN', 'MU (Threshold)');
coefs_car <- c(-4.494, 0.955, 1.054, -0.105, 0.193, 0.409, 0.473, -0.417, -0.258, 0.285, 1.450);

glabels_m6_1 <- c()
gedits_m6_1 <- c()
for (i in 1:(length(coef_names_car))) {
  
  var_name <- coef_names_car[[i]]
  var_value <- coefs_car[[i]]
  
  tmp_label <- glabel(var_name, container = tbl)
  car_var1 <- gedit(var_value, container = tbl)
  size(car_var1) <- c(100, 20);

  #if (i >= length(coef_names_car)){
  gseparator(container = tbl, expand=TRUE)
  #}
  
  tbl[i,1] <- tmp_label
  tbl[i,2] <- car_var1
  
  glabels_m6_1 <- c(glabels_m6_1, c(tmp_label))
  gedits_m6_1 <- c(gedits_m6_1, c(car_var1))
  
}




tmp_group <- ggroup(container = model6, anchor=c(-1,1), expand=TRUE);
tmp_frame <- gframe("6.2 VAN Count", container=tmp_group, horizontal=FALSE, expand=TRUE);

tbl <- glayout(container=tmp_frame)

coef_names_van <- c('const', 'YVAN6_11', 'YVAN12', 'INCOME2', 'vanmile', 'TAZ_den_Q3', 'AUTO10_Q1', 'EMPDEN');
coefs_van <- c(-4.620, 1.952, 2.572, -0.593, 0.00006, 0.597, 0.867, -0.00023);

glabels_m6_2 <- c()
gedits_m6_2 <- c()
for (i in 1:(length(coef_names_van))) {
  
  var_name <- coef_names_van[[i]]
  var_value <- coefs_van[[i]]
  
  tmp_label <- glabel(var_name, container = tbl)
  van_var1 <- gedit(var_value, container = tbl)
  size(van_var1) <- c(100, 20);
  
  tbl[i,1] <- tmp_label
  tbl[i,2] <- van_var1
  
  glabels_m6_2 <- c(glabels_m6_2, c(tmp_label))
  gedits_m6_2 <- c(gedits_m6_2, c(van_var1))
  
}



tmp_group <- ggroup(container = model6, anchor=c(-1,1), expand=TRUE);
tmp_frame <- gframe("6.3 SUV Count", container=tmp_group, horizontal=FALSE, expand=TRUE);

tbl <- glayout(container=tmp_frame)

coef_names_suv <- c('const', 'YSUV0_5', 'YSUV12', 'NUMADLT', 'INCOME2', 'INCOME5', 'suvmile', 'WORK3', 'RATE_TRANSIT30');
coefs_suv <- c(-3.640, 0.905, 1.496, 0.455, -0.535, 0.398, 0.00004, -0.459, -34.163);

glabels_m6_3 <- c()
gedits_m6_3 <- c()
for (i in 1:(length(coef_names_suv))) {
  
  var_name <- coef_names_suv[[i]]
  var_value <- coefs_suv[[i]]
  
  tmp_label <- glabel(var_name, container = tbl)
  suv_var1 <- gedit(var_value, container = tbl)
  size(suv_var1) <- c(100, 20);
  
  tbl[i,1] <- tmp_label
  tbl[i,2] <- suv_var1
  
  glabels_m6_3 <- c(glabels_m6_3, c(tmp_label))
  gedits_m6_3 <- c(gedits_m6_3, c(suv_var1))
  
}



tmp_group <- ggroup(container = model6, anchor=c(-1,1), expand=TRUE);
tmp_frame <- gframe("6.4 Pick-up Truck Count", container=tmp_group, horizontal=FALSE, expand=TRUE);

tbl <- glayout(container=tmp_frame)

coef_names_pick <- c('const', 'YPICK0_5', 'YPICK6_11', 'pickmile', 'INCOME5', 'WORK3', 'OWN', 'empden_per_100', 'TAZ_den_Q3', 'AUTO30_Q1');
coefs_pick <- c(-3.946, 0.869, 1.016, 0.000058, -0.287, 0.460, 0.808, 0.0078, 0.299, 0.268);

glabels_m6_4 <- c()
gedits_m6_4 <- c()
for (i in 1:(length(coef_names_pick))) {
  
  var_name <- coef_names_pick[[i]]
  var_value <- coefs_pick[[i]]
  
  tmp_label <- glabel(var_name, container = tbl)
  pick_var1 <- gedit(var_value, container = tbl)
  size(pick_var1) <- c(100, 20);

  tbl[i,1] <- tmp_label
  tbl[i,2] <- pick_var1
  
  glabels_m6_4 <- c(glabels_m6_4, c(tmp_label))
  gedits_m6_4 <- c(gedits_m6_4, c(pick_var1))

}


check_string <- function(strvalue) {
  tryCatch(
    {
      
      tmp <- 1 * as.numeric(strvalue); 
      #message("Working");
      return(TRUE);
    },
    error=function(cond) {
      #message("Not Working");
      return(FALSE);
    }
  )    
}


load_data <- function(h,...) {
  
  #current_folder <<- "C:/workspace/VFC_Package_Beta/Model_Application/";
  
  # Read Scrambled Halton Sequence numbers 
  #Halton_File <<- "C:/workspace/VFC_Package_Beta/Halton_R/haltbrat_vfc101.csv";
  
  # Input Data
  #data1 <<- read.table(paste(c(current_folder,'az_hhld_vfc_cleaned_final.csv'),collapse=''), header=T, sep=",")
  
  # Output File Name
  #output_file <- "VFC_output.csv";
  
  current_folder <<- svalue(curfolder_var);
  Halton_File <<- svalue(halton_var);
  input_file <- svalue(input_var);
  output_file <- svalue(output_var);
  
  #data1 <<- read.table(paste(c(current_folder,input_file),collapse=''), header=T, sep=",");
  data1 <<- read.table(input_file, header=T, sep=",");
  
  # 1
  print("Running mileage prediction model .......");
  source(paste(c(current_folder,'1_Mileage_prediction.r'),collapse=''));
  
  coefs_name_mile <- c()
  coefs_mile <- c()
  for (i in 1:(length(gedits_m1))) {
    
    tmp1 <- svalue(glabels_m1[[i]])
    
    if (check_string(svalue(gedits_m1[[i]]))) {
      tmp2 <- as.numeric(svalue(gedits_m1[[i]]))
    }
    else {
      stop("An input value is not valid in 1 Mileage Prediction Model.");
    }
    
    coefs_name_mile <- c(coefs_name_mile,c(tmp1))
    coefs_mile <- c(coefs_mile,c(tmp2))
    
  }
  
  tt1 <- mileage_model(data1, coefs_name_mile, coefs_mile);
  data1 <<- cbind(data1,tt1);
  rm(tt1);
  
  
  # 2
  print("Running MNL model for the number of alternatives .......");
  source(paste(c(current_folder,'2_Number_of_alternatives.r'),collapse=''));
  

  coef_names_veh <- list();
  coef_values_veh <- list();
  for(i in 1:length(glabel_list_m2)) {
    
    tmp_glabel_m2 <- glabel_list_m2[[i]];
    tmp_gedit_m2 <- gedit_list_m2[[i]];
    
    coef_names <- c();
    coef_values <- c();
    for(j in 1:length(tmp_glabel_m2)) {
      
      tmp1 <- svalue(tmp_glabel_m2[[j]])
      tmp2 <- as.numeric(svalue(tmp_gedit_m2[[j]]))
      coef_names <- c(coef_names,c(tmp1))
      coef_values <- c(coef_values,c(tmp2))     
    }
    
    coef_names_veh[[i]] <- coef_names;
    coef_values_veh[[i]] <- coef_values;
  }

  coef_names_veh[[(length(glabel_list_m2)+1)]] <- c('const');
  coef_values_veh[[(length(glabel_list_m2)+1)]] <- c(0.0);

  tt1 <- NumAlternative_model(data1, coef_names_veh, coef_values_veh);
  data1 <<- cbind(data1,tt1);
  rm(tt1);

  

  ivmts <- list();
  ivmts_coef <- c();
  for(i in 1:length(glabel_list_m3)) {
    
    tmp_glabel_m3 <- glabel_list_m3[[i]];
    tmp_gedit_m3 <- gedit_list_m3[[i]];
    
    coef_names <- c();
    coef_values <- c();
    for(j in 1:length(tmp_glabel_m3)) {
      
      tmp1 <- svalue(tmp_glabel_m3[[j]])
      tmp2 <- as.numeric(svalue(tmp_gedit_m3[[j]]))
      coef_names <- c(coef_names,c(tmp1))
      coef_values <- c(coef_values,c(tmp2))     
    }
    
    ivmts[[i]] <- coef_names;
    ivmts_coef <- c(ivmts_coef,coef_values);
  }
  

  ivgts <- list();
  ivgts_coef <- c();
  controls <- list();
  for(i in 1:length(mdcev_set_var)) {
    controls[[i]] <- as.numeric(svalue(mdcev_set_var[[i]]));
  }

  for(i in 1:length(glabel_list_m32)) {
    
    tmp_glabel_m32 <- glabel_list_m32[[i]];
    tmp_gedit_m32 <- gedit_list_m32[[i]];
    
    coef_names <- c();
    coef_values <- c();
    for(j in 1:length(tmp_glabel_m32)) {
      
      tmp1 <- svalue(tmp_glabel_m32[[j]]);
      tmp2 <- as.numeric(svalue(tmp_gedit_m32[[j]]));
      coef_names <- c(coef_names,c(tmp1));
      coef_values <- c(coef_values,c(tmp2)) ;    
    }
    
    ivgts[[i]] <- coef_names;
    ivgts_coef <- c(ivgts_coef,coef_values);
  }
  
  # 3
  print("Running MDCEV forecasting code.......");
  source(paste(c(current_folder,'3_Vehicle_fleet_mix.r'),collapse=''));
  
  mdcev <- VehicleFleet_model(data1, ivmts, ivmts_coef, ivgts, ivgts_coef, controls);
  

  
  #####################################################################################################################################################
  # 4
  # Hueristic mileage reallocation algorithm
  Tolerance <<- as.numeric(svalue(gedit_tol)); # 0.035; # Tolerance limit to match predicted bodytype distribution
  max_iteration <<- as.numeric(svalue(gedit_maxiter)) #5; # Maximum number of iterations for the mileage re-allocation algorithm
  nc <<- 14;
  
  diff_body <<- c(1.0, 1.0, 1.0, 1.0, 1.0);
  iter <<- 1;
  
  print("Running heuristic mileage reallocation algorithm....");
  source(paste(c(current_folder,'4_Heuristic_mileage_reallocation.r'),collapse=''));
  
  
  #####################################################################################################################################################
  # 5
  # MNL model for number of vehicle body types
  # Calibrated coefficients
  print("Checking tolerance limits .......");
  source(paste(c(current_folder,'5_Number_of_bodytypes.r'),collapse=''));
 
  
  coef_names_type <- list();
  coef_values_type <- list();
  for(i in 1:length(glabel_list_m5)) {
    
    tmp_glabel_m5 <- glabel_list_m5[[i]];
    tmp_gedit_m5 <- gedit_list_m5[[i]];
    
    coef_names <- c();
    coef_values <- c();
    for(j in 1:length(tmp_glabel_m5)) {
      
      tmp1 <- svalue(tmp_glabel_m5[[j]])
      tmp2 <- as.numeric(svalue(tmp_gedit_m5[[j]]))
      coef_names <- c(coef_names,c(tmp1))
      coef_values <- c(coef_values,c(tmp2))     
    }
    
    coef_names_type[[i]] <- coef_names;
    coef_values_type[[i]] <- coef_values;
  }
  
  #coef_names_type[[(length(glabel_list_m5)+1)]] <- c('const');
  #coef_values_type[[(length(glabel_list_m5)+1)]] <- c(0.0);
  
  
  infor <- Null;
  while ((iter <= max_iteration) & (diff_body[1] > Tolerance | diff_body[2] > Tolerance | diff_body[3] > Tolerance | diff_body[4] > Tolerance | diff_body[5] > Tolerance)) {
    
    print(paste("Iteration", iter));

    
    tt2 <- Heuristic_mileage(data1, mdcev);
    indfor <- tt2[[1]];
    bothnum <- tt2[[2]];
    bothmile <- tt2[[3]];
    NumBodytypes(data1, indfor, bothnum, bothmile, coef_names_type, coef_values_type);

    
    print(diff_body);
    iter <- iter+1;
  }
  
  if(iter > max_iteration){
    print("Consider adjusting the tolerance criteria and/or re-calibrate MDCEV model");
  }
  
  Sys.sleep(3)
  
  
  
  #####################################################################################################################################################
  # 6.1
  # Ordered probit Model for car count
  # Calibrated coefficients

  print("Running count models for CAR .......");
  source(paste(c(current_folder,'6.1_Car_count.r'),collapse=''));
  
  coef_names_car <- c()
  coefs_car <- c()
  for (i in 1:(length(gedits_m6_1))) {
    
    tmp1 <- svalue(glabels_m6_1[[i]])
    tmp2 <- as.numeric(svalue(gedits_m6_1[[i]]))
    coef_names_car <- c(coef_names_car,c(tmp1))
    coefs_car <- c(coefs_car,c(tmp2))
    
  }
  
  tt1 <- car_count_model(data1, indfor, coef_names_car, coefs_car);
  indfor <- cbind(indfor,tt1);
  rm(tt1);
  


  #####################################
  # 6.2
  # Ordered probit model for van count
  # Calibrated coefficients
  
  print("Running count models for VAN .......");
  source(paste(c(current_folder,'6.2_Van_count.r'),collapse=''));
  
  coef_names_van <- c()
  coefs_van <- c()
  for (i in 1:(length(gedits_m6_2))) {
    
    tmp1 <- svalue(glabels_m6_2[[i]])
    tmp2 <- as.numeric(svalue(gedits_m6_2[[i]]))
    coef_names_van <- c(coef_names_van,c(tmp1))
    coefs_van <- c(coefs_van,c(tmp2))
    
  }

  
  tt1 <- van_count_model(data1, indfor, coef_names_van, coefs_van);
  indfor <- cbind(indfor,tt1);
  rm(tt1);  

  
  #####################################
  # 6.3
  # Ordered probit model for SUV count
  # Calibrated coefficients

  print("Running count models for SUV .......");
  source(paste(c(current_folder,'6.3_SUV_count.r'),collapse=''));
  
  coef_names_suv <- c()
  coefs_suv <- c()
  for (i in 1:(length(gedits_m6_3))) {
    
    tmp1 <- svalue(glabels_m6_3[[i]])
    tmp2 <- as.numeric(svalue(gedits_m6_3[[i]]))
    coef_names_suv <- c(coef_names_suv,c(tmp1))
    coefs_suv <- c(coefs_suv,c(tmp2))
    
  }
  
  tt1 <- SUV_count_model(data1, indfor, coef_names_suv, coefs_suv);
  indfor <- cbind(indfor,tt1);
  rm(tt1);
  
  
  #####################################
  # 6.4
  # Ordered probit model for pick-up count
  # Calibrated coefficients
  
  
  print("Running count models for PICKUP .......");
  source(paste(c(current_folder,'6.4_Pickup_count.r'),collapse=''));
  
  
  coef_names_pick <- c()
  coefs_pick <- c()
  for (i in 1:(length(gedits_m6_4))) {
   
    tmp1 <- svalue(glabels_m6_4[[i]])
    tmp2 <- as.numeric(svalue(gedits_m6_4[[i]]))
    coef_names_pick <- c(coef_names_pick,c(tmp1))
    coefs_pick <- c(coefs_pick,c(tmp2))
    
  }
  
  tt1 <- pickup_count_model(data1, indfor, coef_names_pick, coefs_pick);
  indfor <- cbind(indfor,tt1);
  rm(tt1);
  
  
  ### Final result of Vehicle Fleet Composition model 
  #write.table(indfor, file = paste(c(current_folder, output_file),collapse=''), row.names=FALSE, sep=",");
  write.table(indfor, file = output_file, row.names=FALSE, sep=",");
  
  print("Successfully done!.")
  
}

logo_frame <- gframe(container=group, spacing=0, expand=TRUE);
tmp_group <- ggroup(container=logo_frame, spacing=0, expand=TRUE);
gimage(filename="C:/workspace/VFC_Package_Beta/Model_Application/images/asu_logo2.gif", container=tmp_group, anchor=c(1,1));
gimage(filename="C:/workspace/VFC_Package_Beta/Model_Application/images/mag_logo2.gif", container=tmp_group, anchor=c(1,0));
gimage(filename="C:/workspace/VFC_Package_Beta/Model_Application/images/gt_logo2.gif", container=tmp_group, anchor=c(1,-1));



btn_group <- ggroup(container=group, spacing=0);
gbutton("Run", container=btn_group, handler=load_data, anchor=c(1,-1));
gbutton("Cancel", handler = function(h,...) {
  dispose(win)
}, container=btn_group, anchor=c(1,-1));

