# This code predicts the fleet mix owned by a household
# The code is run 25 times and average value is computed for each of the alternatives
# Number of replications can be changed by changing the value of "nrep" in control file
# This code takes 'predicted annual mileage' as input from '1_Mileage_prediction.r'
# Output of this code serves as input to '4_Heuristic_mileage_reallocation.r'
mdcev_ind <- function(ivmt, ind, nc) {
  
  i <- 1;
  while (i <= (nc-1)) {
    j <- 1;
    while (j <= numcoefficients[i]) {
      if (i > ind) {
        ivmt <- c(ivmt, "sero");
      }
      if (i < ind) {
        ivmt <- c("sero", ivmt);
      }
      j <- j+1;
    }
    i <- i+1;
  }
  
  return (ivmt);
  
}


mdcev_ivgt_ind <- function(ivgt, ind, nc) {
  
  ivgt <- c("sero", ivgt);
  i <- 1;
  while (i <= (nc-1)) {
    j <- 1;
    while (j <= numgamma[i]) {
      if (i > ind) {
        ivgt <- c(ivgt, "sero");
      }
      if (i < ind) {
        ivgt <- c("sero", ivgt);
      }
      j <- j+1;
    }
    i <- i+1;
  }
  
  return (ivgt);
  
}


VehicleFleet_model <- function(data1, ivmts, ivmts_coef, ivgts, ivgts_coef, controls) {

  uno <- controls[[1]] #"uno"; # Column of ones
  sero <- controls[[2]] #"sero"; # Column of zeros
  
  po <- controls[[3]];   #7 # Column number of the unique identifier created for a household (this column is created by the user)
  nrep <- controls[[4]]; #25 # Number of replications of the MDCEV model to be carried out
  
  numout <- controls[[5]];  #1
  config <- controls[[6]];  #4    #Utility specification configuration, possible values: 1,4,5,6,7
  alp0to1 <- 1;    #1 if you want the Alpha values to be constrained between 0 and 1, 0 otherwise
  #putting _alp0to1 = 1 is recommended practice and can provide estimation stability
  price <- 0;       #1 if there is price variation across goods, 0 otherwise
  
  #alt_names <- c("NONMOTOR","YCAR0_5","YCAR6_11","YCAR12","YVAN0_5","YVAN6_11","YVAN12","YSUV0_5","YSUV6_11","YSUV12","YPICK0_5","YPICK6_11","YPICK12","MOTOR");
  alt_names <- c("NONMOTOR","CAR1","CAR2","CAR3","VAN1","VAN2","VAN3","SUV1","SUV2","SUV3","PICKUP1","PICKUP2","PICKUP3","MOTOR");
  
  nc <- length(alt_names);  #14;        #Number of alternatives (in the universal choice set) including outside goods
  gumbel <- 1;
  
  
  if (gumbel == 0) {
    nrep <- 1;
  }
  
  
  #tolel <- 0.00000001;
  #tolee <- 0.01;
  avg <<- 1;
  
  
  #ivuno <- "uno";
  #ivsero <- "sero";
  #wtind <<- "uno";
  
  budget_index <- grep('budget', colnames(data1));
  #f <- c(budget_index, 263, 264, 265, 266, 267, 268, 269, 270, 271, 272,  273, 274, 275);
  f <- c(budget_index, sero, sero, sero, sero, sero, sero, sero, sero, sero, sero,  sero, sero, sero);
  
  #fp <- c(5,  5,  5,  5,  5,	5,	5,	5,	5,	5,	5,	5,	5,	5); 
  fp <- c(rep(uno,nc));
  
  # ivmt1 <- c(6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6);
  # ivmt2 <- c(5, 127, 130, 132, 137, 159, 187, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6);
  # ivmt3 <- c(6, 6, 6, 6, 6, 6, 6, 5, 136, 128, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6);
  # ivmt4 <- c(6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 5, 127, 152, 159, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6);
  # ivmt5 <- c(6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 5, 132, 136, 192, 188, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6);
  # ivmt6 <- c(6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 5, 132, 192, 128, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6);
  # ivmt7 <- c(6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 5, 47, 192, 127, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6);
  # ivmt8 <- c(6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 5, 127, 136, 152, 159, 187, 190, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6);
  # ivmt9 <- c(6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 5, 129, 141, 151, 200, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6);
  # ivmt10 <- c(6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 5, 130, 133, 125, 191, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6);
  # ivmt11 <- c(6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 5, 131, 138, 124, 160, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6);
  # ivmt12 <- c(6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 5, 124, 152, 130, 196, 6, 6, 6, 6, 6, 6, 6, 6, 6);
  # ivmt13 <- c(6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 5, 159, 128, 133, 196, 6, 6, 6, 6);
  # ivmt14 <- c(6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 6, 5, 124, 151, 138);
  ivmt1 <- c("sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero", "sero");
  ivmt2 <- c(ivmts[[1]]);
  ivmt3 <- c(ivmts[[2]]);
  ivmt4 <- c(ivmts[[3]]);
  ivmt5 <- c(ivmts[[4]]);
  ivmt6 <- c(ivmts[[5]]);
  ivmt7 <- c(ivmts[[6]]);
  ivmt8 <- c(ivmts[[7]]);
  ivmt9 <- c(ivmts[[8]]);
  ivmt10 <- c(ivmts[[9]]);
  ivmt11 <- c(ivmts[[10]]);
  ivmt12 <- c(ivmts[[11]]);
  ivmt13 <- c(ivmts[[12]]);  
  ivmt14 <- c(ivmts[[13]]);
  
  numcoefficients <<- c(length(ivmt2), length(ivmt3), length(ivmt4), length(ivmt5), length(ivmt6), length(ivmt7), length(ivmt8),
                       length(ivmt9), length(ivmt10), length(ivmt11), length(ivmt12), length(ivmt13), length(ivmt14));

  
  ivmt2 <- mdcev_ind(ivmt2, 1, nc);
  ivmt3 <- mdcev_ind(ivmt3, 2, nc);
  ivmt4 <- mdcev_ind(ivmt4, 3, nc);
  ivmt5 <- mdcev_ind(ivmt5, 4, nc);
  ivmt6 <- mdcev_ind(ivmt6, 5, nc);
  ivmt7 <- mdcev_ind(ivmt7, 6, nc);
  ivmt8 <- mdcev_ind(ivmt8, 7, nc);
  ivmt9 <- mdcev_ind(ivmt9, 8, nc);
  ivmt10 <- mdcev_ind(ivmt10, 9, nc);
  ivmt11 <- mdcev_ind(ivmt11, 10, nc);
  ivmt12 <- mdcev_ind(ivmt12, 11, nc);
  ivmt13 <- mdcev_ind(ivmt13, 12, nc);
  ivmt14 <- mdcev_ind(ivmt14, 13, nc);
  
  
  
  #First good is outside good "Annual Non_auto mileage"
  # ivdt1 <-	c(5,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6);
  # ivdt2	<-	c(6,	5,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6);
  # ivdt3	<-	c(6,	6,	5,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6);
  # ivdt4	<-	c(6,	6,	6,	5,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6);
  # ivdt5	<-	c(6,	6,	6,	6,	5,	6,	6,	6,	6,	6,	6,	6,	6,	6);
  # ivdt6	<-	c(6,	6,	6,	6,	6,	5,	6,	6,	6,	6,	6,	6,	6,	6);
  # ivdt7	<-	c(6,	6,	6,	6,	6,	6,	5,	6,	6,	6,	6,	6,	6,	6);
  # ivdt8	<-	c(6,	6,	6,	6,	6,	6,	6,	5,	6,	6,	6,	6,	6,	6);
  # ivdt9	<-	c(6,	6,	6,	6,	6,	6,	6,	6,	5,	6,	6,	6,	6,	6);
  # ivdt10 <-	c(6,	6,	6,	6,	6,	6,	6,	6,	6,	5,	6,	6,	6,	6);
  # ivdt11 <-	c(6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	5,	6,	6,	6);
  # ivdt12 <-	c(6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	5,	6,	6);
  # ivdt13 <-	c(6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	5,	6);
  # ivdt14 <-	c(6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	5);
  
  
  ivdt1 <-  c(uno,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero);
  ivdt2	<-	c(sero,	uno,	sero, sero,	sero,	sero,	sero,	sero,	sero,  sero,	sero,	sero,	sero,	sero);
  ivdt3	<-	c(sero,	sero,	uno,	sero,	sero,	sero,	sero,	sero,	sero,  sero,	sero,	sero,	sero,	sero);
  ivdt4	<-	c(sero,	sero,	sero,	uno,	sero,	sero,	sero,	sero,	sero,  sero,	sero,	sero,	sero,	sero);
  ivdt5	<-	c(sero,	sero,	sero,	sero,	uno,	sero,	sero,	sero,	sero,  sero,	sero,	sero,	sero,	sero);
  ivdt6	<-	c(sero,	sero,	sero,	sero,	sero,	uno,	sero,	sero,	sero,  sero,	sero,	sero,	sero,	sero);
  ivdt7	<-	c(sero,	sero,	sero,	sero,	sero,	sero,	uno,	sero,	sero,  sero,	sero,	sero,	sero,	sero);
  ivdt8	<-	c(sero,	sero,	sero,	sero,	sero,	sero,	sero,	uno,	sero,	sero,	sero,	sero,	sero,	sero);
  ivdt9	<-	c(sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	uno,	sero,	sero,	sero,	sero,	sero);
  ivdt10 <-	c(sero,  sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	uno,	sero,	sero,	sero,	sero);
  ivdt11 <-	c(sero,  sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	uno,	sero,	sero,	sero);
  ivdt12 <-	c(sero,  sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	uno,	sero,	sero);
  ivdt13 <-	c(sero,  sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	uno,	sero);
  ivdt14 <-	c(sero,  sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	sero,	uno);
  
  
  #First good is outside good "home epoch" and therefore gamma = 0 for ivgt1
  # ivgt1 <- c(6,  6,  6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6);
  # ivgt2 <- c(6,	5,	127, 128, 129, 6, 6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6);
  # ivgt3 <- c(6,	6,	6,	6,	6,	5, 127,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6);
  # ivgt4 <- c(6,	6,	6,	6,	6,	6,	6,	5,	127, 6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6);
  # ivgt5 <- c(6,	6,	6,	6,	6,	6,	6,	6,	6,	5,	127, 128, 6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6);
  # ivgt6 <- c(6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	5,	128, 129, 6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6);
  # ivgt7 <- c(6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	5,	127, 128, 129, 6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6);
  # ivgt8 <- c(6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	5,	127, 128, 6,	6,	6,	6,	6,	6,	6,	6,	6);
  # ivgt9 <- c(6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	5,	6,	6,	6,	6,	6,	6,	6,	6);
  # ivgt10 <- c(6,	6,	6, 6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	5,	6,	6,	6,	6,	6,	6,	6);
  # ivgt11 <- c(6,	6,	6, 6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	5,	6,	6,	6,	6,	6,	6);
  # ivgt12 <- c(6,	6,	6, 6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	5,	6,	6,	6,	6,	6);
  # ivgt13 <- c(6,	6,	6, 6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	5,	6,	6,	6,	6);
  # ivgt14 <- c(6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	6,	5, 127, 128, 129);
  ivgt1 <- c("sero",  "sero",  "sero",  "sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero",	"sero");

  ivgt2 <- c(ivgts[[1]]);
  ivgt3 <- c(ivgts[[2]]);
  ivgt4 <- c(ivgts[[3]]);
  ivgt5 <- c(ivgts[[4]]);
  ivgt6 <- c(ivgts[[5]]);
  ivgt7 <- c(ivgts[[6]]);
  ivgt8 <- c(ivgts[[7]]);
  ivgt9 <- c(ivgts[[8]]);
  ivgt10 <- c(ivgts[[9]]);
  ivgt11 <- c(ivgts[[10]]);
  ivgt12 <- c(ivgts[[11]]);
  ivgt13 <- c(ivgts[[12]]);  
  ivgt14 <- c(ivgts[[13]]);
  
  
  numgamma <<- c(length(ivgt2), length(ivgt3), length(ivgt4), length(ivgt5), length(ivgt6), length(ivgt7), length(ivgt8),
                length(ivgt9), length(ivgt10), length(ivgt11), length(ivgt12), length(ivgt13), length(ivgt14));
  
  ivgt2 <- mdcev_ivgt_ind(ivgt2, 1, nc);
  ivgt3 <- mdcev_ivgt_ind(ivgt3, 2, nc);
  ivgt4 <- mdcev_ivgt_ind(ivgt4, 3, nc);
  ivgt5 <- mdcev_ivgt_ind(ivgt5, 4, nc);
  ivgt6 <- mdcev_ivgt_ind(ivgt6, 5, nc);
  ivgt7 <- mdcev_ivgt_ind(ivgt7, 6, nc);
  ivgt8 <- mdcev_ivgt_ind(ivgt8, 7, nc);
  ivgt9 <- mdcev_ivgt_ind(ivgt9, 8, nc);
  ivgt10 <- mdcev_ivgt_ind(ivgt10, 9, nc);
  ivgt11 <- mdcev_ivgt_ind(ivgt11, 10, nc);
  ivgt12 <- mdcev_ivgt_ind(ivgt12, 11, nc);
  ivgt13 <- mdcev_ivgt_ind(ivgt13, 12, nc);
  ivgt14 <- mdcev_ivgt_ind(ivgt14, 13, nc);
  
  
  
  ivm <<- c(ivmt1, ivmt2, ivmt3, ivmt4, ivmt5, ivmt6, ivmt7, ivmt8, ivmt9, ivmt10, ivmt11, ivmt12, ivmt13, ivmt14);
  ivd <<- c(ivdt1, ivdt2, ivdt3, ivdt4, ivdt5, ivdt6, ivdt7, ivdt8, ivdt9, ivdt10, ivdt11, ivdt12, ivdt13, ivdt14);
  ivg <<- c(ivgt1, ivgt2, ivgt3, ivgt4, ivgt5, ivgt6, ivgt7, ivgt8, ivgt9, ivgt10, ivgt11, ivgt12, ivgt13, ivgt14);
  
  
  nvarm <<- length(ivmt1);    #number of variables in baseline utility   = number of columns in ivm1, do not modify this
  nvardel <<- length(ivdt1);  #number of variables in satiation          = number of columns in ivd1, do not modify this
  nvargam <<- length(ivgt1);  #number of variables in translation        = number of columns in ivg1, do not modify this
  
  
  #Associating columns with variable names
  flagchm <<- f;        
  #flagavm <<- c(5,  5,  5,  5,	5,	5,	5,	5,	5,	5,	5,	5,	5,	5);
  flagavm <<- c(rep(uno,nc));
  flagprcm <<- fp;
  
  
  #Do not modify the line below####################### 
  if (config == 1) {
    eqmatdel <- diag(nvardel);
    eqmatgam <- rbind(matrix(1,nrow=1,ncol=numout),matrix(0,nrow=1,ncol=numout));
    eqmatgam <- cbind(eqmatgam, rbind(matrix(0,nrow=1,ncol=(nc-numout)),matrix(1,nrow=1,ncol=(nc-numout))));
  }
  if (config == 4) {
    eqmatdel <- matrix(1,nrow=1,ncol=nc);
    eqmatgam <- diag(nvargam);
  }
  if (config == 5) {
    eqmatdel <- matrix(1,nrow=1,ncol=nvardel);
    eqmatgam <- diag(nvargam);
  }
  if (config == 6) {
    eqmatdel <- rbind(matrix(1,nrow=1,ncol=numout),matrix(0,nrow=1,ncol=numout));
    eqmatdel <- cbind(eqmatdel, rbind(matrix(0,nrow=1,ncol=(nc-numout)),matrix(1,nrow=1,ncol=(nc-numout))));
    eqmatgam <- diag(nc);
  }
  if (config == 7) {
    eqmatdel <- matrix(1,nrow=1,ncol=nc);
    eqmatgam <- diag(nc);
  }
  
  
  #mcoef <- c(mcoef2, mcoef3, mcoef4, mcoef5, mcoef6, mcoef7, mcoef8, mcoef9, mcoef10, mcoef11, mcoef12, mcoef13, mcoef14, -1000.0, -1000.0);
  #gcoef <- c(gcoef2, gcoef3, gcoef4, gcoef5, gcoef6, gcoef7, gcoef8, gcoef9, gcoef10, gcoef11, gcoef12, gcoef13, gcoef14, 1.0);
  mcoef <- c(ivmts_coef, -1000.0, -1000.0);
  gcoef <- c(ivgts_coef, 1.0);
  bmdcev <- matrix(c(mcoef, gcoef), ncol=1);
  
  
  lpr <- function(x) {
    
    ptm1 <- proc.time();
    
    e1 <- nrow(data1);
    popass <- as.matrix(data1[,po]);
    
    xdel <- t(eqmatdel)%*%x[(nvarm+1):(nvarm+nrow(eqmatdel)),];
    xgam <- t(eqmatgam)%*%x[(nvarm+nrow(eqmatdel)+1):(nvarm+nrow(eqmatdel)+nrow(eqmatgam)),];
    xsigm <- x[(nvarm+nrow(eqmatdel)+nrow(eqmatgam)+1),];
    
    a <- matrix(c(rep(1,nc)), ncol=1)%x%x[1:nvarm,];
    b <- matrix(c(rep(1,nc)), ncol=1)%x%xdel;
    c <- matrix(c(rep(1,nc)), ncol=1)%x%xgam;
    
    v2 <- sweep(data1[,ivm],MARGIN=2,t(a),'*');
    w2 <- sweep(data1[,ivd],MARGIN=2,t(b),'*');
    u2 <- sweep(data1[,ivg],MARGIN=2,t(c),'*');
    rm(a, b, c);
    
    
    v <- NULL;
    w <- NULL;
    u <- NULL;
    alts <- matrix(0,nrow=1,ncol=nc);
    for (j in 1:nc) {
      #v <- cbind(v, as.matrix(apply(v2[,((j-1)*nvarm+1):(j*nvarm)],1,sum),ncol=1));
      #w <- cbind(w, as.matrix(apply(w2[,((j-1)*nvardel+1):(j*nvardel)],1,sum),ncol=1));
      #u <- cbind(u, as.matrix(apply(u2[,((j-1)*nvargam+1):(j*nvargam)],1,sum),ncol=1));
      
      v <- cbind(v, as.matrix(rowSums(v2[,((j-1)*nvarm+1):(j*nvarm)]),ncol=1));
      w <- cbind(w, as.matrix(rowSums(w2[,((j-1)*nvardel+1):(j*nvardel)]),ncol=1));
      u <- cbind(u, as.matrix(rowSums(u2[,((j-1)*nvargam+1):(j*nvargam)]),ncol=1));
      
      alts[1,j] <- j;
    }
    u[,1:numout] <- -1000*matrix(1,nrow=e1,ncol=numout);
    rm(v2, w2);
    v <- exp(v);
    v <- kronecker(v,matrix(1,nrow=nrep,ncol=1));
    
    if (gumbel == 1) {
      #as <- halton(nrep*e1, dim = nc, init = TRUE, normal = FALSE, usetime = FALSE)
      as <- read.table(Halton_File, header=F, sep=",");
      as <- as.matrix(as[22:(e1*nrep+21),1:nc]);
      as <- -log(-log(as))*xsigm;
    }
    else {
      as <- matrix(0,nrow=(nrep*e1),ncol=nc);
    }
    
    
    v <- v*exp(as);
    tmp <- kronecker(as.matrix(data1[,flagprcm]),matrix(1,nrow=nrep,ncol=1));
    v <- v/tmp ;   # Price-normalized Baseline Utilities
    rm(tmp);
    a <- 1-(1/(1+exp(w))) ;        # Alphas
    prices <- as.matrix(data1[,flagprcm]);
    f1 <- exp(u);                # Gammas  
    
    
    
    for (i in 1:e1) {
      for (j in 1:nrep) {
        fc <- matrix(0,nrow=1,ncol=nc);
        vqr <- rbind(alts,rbind(v[((i-1)*nrep+j),],rbind(prices[i,],f1[i,])));
        vqr1 <- vqr[,1:numout];
        vqr2 <- vqr[,(numout+1):nc];
        vqr <- cbind(vqr1,vqr2[,order(vqr2[2,],decreasing = TRUE)]);
        
        m <- numout;
        k <- -1;
        N <- rowSums(t(prices[i,1:numout]*((vqr[2,1:numout])^(1/(matrix(1,nrow=1,ncol=numout)-a[1,1:numout])))));
        D <- colSums(t(data1[i,flagchm]));
        
        lambda <- (N/D)^(1-a[1,1]);     
        
        if (vqr[2,(numout+1)] < lambda) {
          fc[1,1:numout] <- ((vqr[2,1:numout]/lambda)^(1/(matrix(1,nrow=1,ncol=numout) - a[1,1:numout])));
          fc[1,(numout+1):nc] <- matrix(0,nrow=1,ncol=(nc-1));
        }
        else{
          while (m != k){
            
            m <- m+1;
            if (m == nc) {            
              fc[1,1:numout] <- ((vqr[2,1:numout]/lambda)^(1/(1-a[1,1:numout])));
              fc[1,(numout+1):nc] <- (((vqr[2,(numout+1):nc]/lambda)^(1/(1-a[1,(numout+1):nc])))-matrix(1,nrow=1,ncol=(nc-numout)))*(vqr[4,(numout+1):nc]);
              fc[1,1:nc] <- colSums(t(Data[i,flagchm]))*fc[1,1:nc]/rowSums(t(fc[1,1:nc]));
              k <- m;
            }
            else if (m < nc){
              N <- N + (vqr[4,m]*vqr[3,m]*(vqr[2,m]^(1-a[1,1])));
              D <- D + (vqr[4,m]*vqr[3,m]);
              lambda <- (N/D)^(1-a[1,1]);
              
              if (vqr[2,m+1] < lambda){
                fc[1,1:numout] <- ((vqr[2,1:numout]/lambda)^(1/(1-a[1,1:numout])));
                fc[1,(numout+1):m] <- (((vqr[2,(numout+1):m]/lambda)^(1/(1-a[1,(numout+1):m])))-matrix(1,nrow=1,ncol=(m-numout)))*(vqr[4,(numout+1):m]);
                fc[1,(m+1):nc] <- matrix(0,nrow=1,ncol=(nc-m));
                k <- m;
              }
            }
          }
          
        }
        
        vqr[2,1:nc] <- fc;
        vqr <- vqr[,order(vqr[1,])];
        v[((i-1)*nrep+j),1:nc] <- vqr[2,];
      }
      
    }
    nreps = NULL;
    for (i in 1:nrep) {
      nreps = rbind(nreps, i);
    }
    
    z1 <- kronecker(popass,matrix(1,nrow=nrep,ncol=1));
    z2 <- kronecker(matrix(1,nrow=nrow(popass),ncol=1),nreps);
    z <- cbind(z1,z2);
    z <- cbind(z,v);
    rm(z1,z2);
    return(z);
    
  }
  
  
  Forecasts <- lpr(bmdcev);
  nobs <- nrow(data1);
  if (avg == 1) {
    
    mdcev <- NULL
    for (ii in 0:(nobs-1)) {
      mdcev <- rbind(mdcev,colSums(Forecasts[(ii*nrep+1):(ii*nrep+nrep),]/nrep)) 
    }
    mdcev <- as.data.frame(mdcev); 
    names(mdcev) <- c("ID","AVG",alt_names);
    print("Done!")
  }
  if (avg == 0) {
    print("Done!")
  }
  rm(Forecasts);
  
  observed <- data.frame(data1$annual_non_motor_mileage_new, data1$CAR1, data1$CAR2, data1$CAR3, data1$VAN1, data1$VAN2, data1$VAN3, data1$SUV1, 
                         data1$SUV2, data1$SUV3, data1$PICK1, data1$PICK2, data1$PICK3, data1$MOTOR);
  
  NumHousehold <- nrow(data1);
  pnumbyalter <- mdcev;
  pnumbyalter[pnumbyalter > 0] <- 1;
  pnumbyalter <- pnumbyalter[,3:(ncol(pnumbyalter))];
  
  pavgbyalter <- mdcev;
  pavgbyalter <- colSums(pavgbyalter[,3:(ncol(pavgbyalter))])
  pavgmileage <- pavgbyalter / NumHousehold;
  
  onumbyalter <- observed;
  onumbyalter[onumbyalter > 0] <- 1;
  
  oavgbyalter <- observed;
  oavgbyalter <- colSums(oavgbyalter);
  oavgmileage <- oavgbyalter / NumHousehold;
  
  bothmile <- matrix(0,nrow=2,ncol=ncol(onumbyalter));
  i <- 1;
  while (i <= ncol(onumbyalter)) {
    bothmile[1, i] <- oavgmileage[i];
    bothmile[2, i] <- pavgmileage[i];
    i <- i + 1;
  }
  
  ### Validation chart
  barplot(bothmile, main="Average Annual Mileage before Heuristic Reallocation", legend.text = c("Observed", "Predicted"), xlab="Vehicle Alternatives",  
          ylab="Mileage", names.arg=c("Non Motorized Vehicle","CAR0-5","CAR6-11","CAR>=12","VAN0-5","VAN6-11","VAN>=12","SUV0-5","SUV6-11","SUV>=12","PICK0-5","PICK6-11",
                                      "PICK>=12","Motor Bike"), col = c("blue", "red"), ylim=c(0,6000), beside=TRUE)
  
  return(mdcev);
}

