# This code predicts the number of different body types owned by a household  for any given horizon year and compares it to the bodytype distribution 
# provided by the hueristic mileage reallocation algorithm
# This code takes input from '4_Heuristic_mileage_reallocation.r' and checks it against a predicted body type distribution for user set tolerance limits


NumBodytypes <- function(data1, indfor, bothnum, bothmile, coefs_names, coefs_type) {

  
  
  pst <- proc.time()
  util_internal <- function(names, coefs) {
    
    util_veh <- data1[names[1]] * coefs[1];
    if (length(names) >= 2) {
      for (i in 2:(length(names))) {
        util_veh <- util_veh + data1[names[i]] * coefs[i];
      }
    }
    util_veh <- as.numeric(util_veh$const);
    return (exp(util_veh));
  }
  
  
  coefs_name_0type <- c(coefs_names[[1]]);
  coefs_name_1type <- c(coefs_names[[2]]);
  coefs_name_2type <- c(coefs_names[[3]]);
  coefs_name_3type <- c(coefs_names[[4]]);
  coefs_name_4type <- c(coefs_names[[5]]);
  
  coefs_0type <- c(coefs_type[[1]]);
  coefs_1type <- c(coefs_type[[2]]);
  coefs_2type <- c(coefs_type[[3]]);
  coefs_3type <- c(coefs_type[[4]]);
  coefs_4type <- c(coefs_type[[5]]);
  
  #data1$const <- 1;
  
  num_body <- 4;
  
  ### Utility Computation
  util_zero_body <- util_internal(coefs_name_0type, coefs_0type);
  util_one_body <- util_internal(coefs_name_1type, coefs_1type);
  util_two_body <- util_internal(coefs_name_2type, coefs_2type);
  util_three_body <- util_internal(coefs_name_3type, coefs_3type);
  util_four_body <- util_internal(coefs_name_4type, coefs_4type);
  
  
  ### Probability computation
  total_util <- util_zero_body + util_one_body + util_two_body + util_three_body + util_four_body;
  prob_zero_body <- (util_zero_body/total_util);
  prob_one_body <- (util_one_body/total_util);
  prob_two_body <- (util_two_body/total_util);
  prob_three_body <- (util_three_body/total_util);
  prob_four_body <- (util_four_body/total_util);
  
  
  ### Cumulative probability computation
  cum_prob_zero_body <- prob_zero_body;
  cum_prob_one_body <- cum_prob_zero_body +  prob_one_body;
  cum_prob_two_body <- cum_prob_one_body +  prob_two_body;
  cum_prob_three_body <- cum_prob_two_body +  prob_three_body;
  cum_prob_four_body <- cum_prob_three_body +  prob_four_body; 
  
  
  NumHousehold <- nrow(data1);
  rnum <- runif(NumHousehold);
  rnum[rnum < cum_prob_zero_body] <- 2;
  rnum[rnum < cum_prob_one_body] <- 3;
  rnum[rnum < cum_prob_two_body] <- 4;
  rnum[rnum < cum_prob_three_body] <- 5;
  rnum[rnum <= 1] <- 6;
  data1$pred_body_MNL <- rnum - 2;
  
  
  pnumbybody <- indfor;
  pnumbybody[pnumbybody > 0] <- 1;
  pnumbybody <- pnumbybody[,3:(ncol(pnumbybody))];
  
  
  isCAR <- rowSums(pnumbybody[,1:3]);
  isVAN <- rowSums(pnumbybody[,4:6]);
  isSUV <- rowSums(pnumbybody[,7:9]);
  isPICK <- rowSums(pnumbybody[,10:12]);
  isMOTOR <- pnumbybody[,13];
  isCAR[isCAR > 0] <- 1;
  isVAN[isVAN > 0] <- 1;
  isSUV[isSUV > 0] <- 1;
  isPICK[isPICK > 0] <- 1;
  data1$pred_body_HMR <- isCAR + isVAN + isSUV + isPICK + isMOTOR;
  data1$pred_body_HMR[data1$pred_body_HMR > 4] <- 4;
  
  i <- 1;
  while (i <= 5) {
    diff_body[i] <<- abs((sum(data1$pred_body_HMR == (i-1)) - sum(data1$pred_body_MNL == (i-1)))/NumHousehold);
    i <- i+1;
  }
  
  if ((iter >= max_iteration) | (diff_body[1] <= Tolerance & diff_body[2] <= Tolerance & diff_body[3] <= Tolerance & diff_body[4] <= Tolerance & diff_body[5] <= Tolerance)) {
    
    comp_obs_pred <- list(data1$pred_body_MNL, data1$pred_body_HMR);
    
    # Comparion of body type distribution (predicion of MNL model vs. Output of HMR algorithm)
    multhist(comp_obs_pred, xlab="Number of Vehicle Body Types", ylab="Percent of Households", col = c("blue", "red"),  main = "Body Type Distribution",
             legend.text = c("Predicted by MNL", "Output from HMR"), names.arg=c("0","1","2","3",">=4"), breaks=seq(-0.5,5, by = 1), ylim=c(0,0.6), freq = FALSE); 
    
    
    # Vehicle type distribution comparison
    barplot(bothnum, main="Vehicle Type Distribution", legend.text = c("Observed", "Predicted"),xlab="Vehicle Alternatives",  
            ylab="Percent", names.arg=c("Non Motorized Vehicle","CAR0-5","CAR6-11","CAR>=12","VAN0-5","VAN6-11","VAN>=12","SUV0-5","SUV6-11","SUV>=12","PICK0-5","PICK6-11",
                                        "PICK>=12","Motor Bike"), col = c("blue", "red"), beside=TRUE);
    
    
    # Average annual mileage comparison
    barplot(bothmile, main="Average Annual Mileage (Excluding Zero Mileage Households)", legend.text = c("Observed", "Predicted"), xlab="Vehicle Alternatives",  
            ylab="Mileage", names.arg=c("Non Motorized Vehicle","CAR0-5","CAR6-11","CAR>=12","VAN0-5","VAN6-11","VAN>=12","SUV0-5","SUV6-11","SUV>=12","PICK0-5","PICK6-11",
                                        "PICK>=12","Motor Bike"), col = c("blue", "red"), ylim=c(0,18000), beside=TRUE)
  }

  return(data1[c("pred_body_HMR","pred_body_MNL")]);
}









