# This code predicts the count of cars for each bodytype-age category in a household 
# This code takes input from '4_Heuristic_mileage_reallocation.r'
# Output of code is written to 'VFC_output.csv'


### Function for Monte Carlo Simulation 
count_car <- function(const, expect_op, thresh, ncar) {
  
  p1 <- pnorm(0-expect_op);
  p2 <- pnorm(thresh-expect_op);
  p3 <- const;

  isCAR <- ncar;
  isCAR[isCAR > 0] <- 1;
  

  rnum <- runif(length(const));
  
  
  rnum[rnum < p1] <- 1;
  rnum[rnum < p2] <- 2;
  rnum[rnum < 1] <- 3;
  rnum <- rnum*isCAR;
    
  return (rnum);
}


car_count_model <- function(data1, indfor, coef_names, coef_values) {

  
  ### Dummy variables for Ordered Probit model 
  data1$YCAR0_5 <- indfor$CAR1
  data1$YCAR0_5 [data1$YCAR0_5 > 0] <- 1
  data1$YCAR6_11 <- indfor$CAR2
  data1$YCAR6_11 [data1$YCAR6_11 > 0] <- 1
  data1$YCAR12 <- indfor$CAR3
  data1$YCAR12 [data1$YCAR12 > 0] <- 1
  
  ### Compute total predicted mileage for car in the level of household
  data1$carmile <- indfor$CAR1 + indfor$CAR2 + indfor$CAR3;
  
  ### Observed car counts
  data1$ncar1 <- data1$NCAR
  data1$ncar1 [data1$NCAR >= 4] <- 3
  data1$ncar2 <- data1$ncar1 - 1
  
  data1$obs_ncar <- data1$ncar1 
  data1$ncar2 <- factor(data1$ncar2)

  
  ### Ordered probit utility computation for car
  #data1$const <- 1
  data1$cons <- 1*coef_values[1]*data1$const

  expect_op <- data1[coef_names[1]] * coef_values[1]
  for (i in 2:(length(coef_names) - 1)) {
    expect_op <- expect_op + data1[coef_names[i]] * coef_values[i]
  }

  expect_op <- as.numeric(expect_op$const)
  thresh <- coef_values[length(coef_values)]
  
  
  ### Monte Carlo simulation for each age category
  indfor$NCAR1 <- count_car(data1$const, expect_op, thresh, indfor$CAR1);
  indfor$NCAR2 <- count_car(data1$const, expect_op, thresh, indfor$CAR2);
  indfor$NCAR3 <- count_car(data1$const, expect_op, thresh, indfor$CAR3);
  
  
  data1$pred_ncar <- indfor$NCAR1 + indfor$NCAR2 + indfor$NCAR3;
  data1$pred_ncar[data1$pred_ncar > 3] <- 3;
  comp_obs_pred <- list(data1$obs_ncar, data1$pred_ncar)
  
  
  ### Chart showing observed vs. predicted car counts
  multhist(comp_obs_pred, xlab="Number of Cars in Household", ylab="Percent of Households", col = c("blue", "red"),  main = "Car Count",
           legend.text = c("Observed", "Predicted"), breaks=seq(-0.5,3.5, by = 1), names.arg=c("0","1","2",">=3"), ylim=c(0,1),  freq = FALSE) 
  
  return(indfor[c("NCAR1","NCAR2","NCAR3")])
}
