# This code predicts the count of vans for each bodytype-age category in a household 
# This code takes inputs from '4_Heuristic_mileage_reallocation.r'
# Output of code is written to 'VFC_output.csv'


### Function for Monte Carlo Simulation 
count_veh <- function(const, expect_op, nveh) {
  
  p1 <- pnorm(0-expect_op);
  p2 <- const;
  
  isVEH <- nveh;
  isVEH[isVEH > 0] <- 1;
  
  
  rnum <- runif(length(const));
  
  
  rnum[rnum < p1] <- 1;
  rnum[rnum < 1] <- 2;
  rnum <- rnum*isVEH;
  
  return (rnum);
}


van_count_model <- function(data1, indfor, coef_names, coef_values) {
  
  
  ### Dummy variables for Ordered Probit model 
  data1$YVAN0_5 <- indfor$VAN1;
  data1$YVAN0_5 [data1$YVAN0_5 > 0] <- 1;
  data1$YVAN6_11 <- indfor$VAN2;
  data1$YVAN6_11 [data1$YVAN6_11 > 0] <- 1;
  data1$YVAN12 <- indfor$VAN3;
  data1$YVAN12 [data1$YVAN12 > 0] <- 1;
  data1$vanmile <- indfor$VAN1 + indfor$VAN2 + indfor$VAN3;
  
  ### Observed van counts
  data1$nvan1 <- data1$NVAN;
  data1$nvan1 [data1$NVAN >= 3] <- 2;
  data1$nvan2 <- data1$nvan1 - 1;
  data1$obs_nvan <- data1$nvan1;
  data1$nvan2 <- factor(data1$nvan2);
  
  
  ### Ordered probit utility computation for van
  #data1$const <- 1;
  data1$cons <- coef_values[1]*data1$const;
  
  expect_op <- data1[coef_names[1]] * coef_values[1];
  for (i in 2:(length(coef_names))) {
    expect_op <- expect_op + data1[coef_names[i]] * coef_values[i];
  }
  expect_op <- as.numeric(expect_op$const);
  
  
  ### Monte Carlo simulation for each age category
  indfor$NVAN1 <- count_veh(data1$const, expect_op, indfor$VAN1);
  indfor$NVAN2 <- count_veh(data1$const, expect_op, indfor$VAN2);
  indfor$NVAN3 <- count_veh(data1$const, expect_op, indfor$VAN3);
  
  
  data1$pred_nvan <- indfor$NVAN1 + indfor$NVAN2 + indfor$NVAN3;
  data1$pred_nvan[data1$pred_nvan > 2] <- 2;
  comp_obs_pred <- list(data1$obs_nvan, data1$pred_nvan)
  
  
  ### Chart showing observed vs. predicted van counts
  multhist(comp_obs_pred, xlab="Number of Vans in Household", ylab="Percent of Households", col = c("blue", "red"),  main = "Van Count",
           legend.text = c("Observed", "Predicted"), breaks=seq(-0.5,3, by = 1), names.arg=c("0","1",">=2"), ylim=c(0,1),  freq = FALSE);
  
  return(indfor[c("NVAN1","NVAN2","NVAN3")]);
}



