# This code predicts the count of SUVs for each bodytype-age category in a household 
# This code takes inputs from '4_Heuristic_mileage_reallocation.r'
# Output of code is written to 'VFC_output.csv'

### Monte Carlo Simulation code used from '6.2_Van_count.r'

SUV_count_model <- function(data1, indfor, coef_names, coef_values) {
 
  ### Dummy variables for Ordered Probit model 
  data1$YSUV0_5 <- indfor$SUV1;
  data1$YSUV0_5 [data1$YSUV0_5 > 0] <- 1;
  data1$YSUV6_11 <- indfor$SUV2;
  data1$YSUV6_11 [data1$YSUV6_11 > 0] <- 1;
  data1$YSUV12 <- indfor$SUV3;
  data1$YSUV12 [data1$YSUV12 > 0] <- 1;
  data1$suvmile <- indfor$SUV1 + indfor$SUV2 + indfor$SUV3;
  
  ### Observed SUV counts
  data1$nsuv1 <- data1$NSUV;
  data1$nsuv1 [data1$NSUV >= 3] <- 2;
  data1$nsuv2 <- data1$nsuv1 - 1;
  data1$obs_nsuv <- data1$nsuv1;
  data1$nsuv2 <- factor(data1$nsuv2);
  
  
  ### Ordered probit utility computation for SUV
  #data1$const <- 1;
  data1$cons <- coef_values[1]*data1$const;
  
  expect_op <- data1[coef_names[1]] * coef_values[1];
  for (i in 2:(length(coef_names))) {
    expect_op <- expect_op + data1[coef_names[i]] * coef_values[i];
  }
  expect_op <- as.numeric(expect_op$const);
  
  
  ### Monte Carlo simulation for each age category
  indfor$NSUV1 <- count_veh(data1$const, expect_op, indfor$SUV1);
  indfor$NSUV2 <- count_veh(data1$const, expect_op, indfor$SUV2);
  indfor$NSUV3 <- count_veh(data1$const, expect_op, indfor$SUV3);
  
  
  data1$pred_nsuv <- indfor$NSUV1 + indfor$NSUV2 + indfor$NSUV3;
  data1$pred_nsuv[data1$pred_nsuv > 2] <- 2;
  comp_obs_pred <- list(data1$obs_nsuv, data1$pred_nsuv);
  
  ### Chart showing observed vs. predicted SUV counts
  multhist(comp_obs_pred, xlab="Number of SUVs in Household", ylab="Percent of Households", col = c("blue", "red"),  main = "SUV Count",
           legend.text = c("Observed", "Predicted"), breaks=seq(-0.5,3, by = 1), names.arg=c("0","1",">=2"), ylim=c(0,1),  freq = FALSE);
  
  
  return (indfor[c("NSUV1","NSUV2","NSUV3")]);
}

