# This code predicts the count of Pick-ups for each bodytype-age category in a household 
# This code takes inputs from '4_Heuristic_mileage_reallocation.r'
# Output of code is written to 'VFC_output.csv'

### Monte Carlo Simulation code used from '6.2_Van_count.r' 

pickup_count_model <- function(data1, indfor, coef_names, coef_values) {

  ### Dummy variables for Ordered Probit model 
  data1$YPICK0_5 <- indfor$PICKUP1
  data1$YPICK0_5 [data1$YPICK0_5 > 0] <- 1
  data1$YPICK6_11 <- indfor$PICKUP2
  data1$YPICK6_11 [data1$YPICK6_11 > 0] <- 1
  data1$YPICK12 <- indfor$PICKUP3
  data1$YPICK12 [data1$YPICK12 > 0] <- 1
  data1$pickmile <- indfor$PICKUP1 + indfor$PICKUP2 + indfor$PICKUP3;
  
  
  ### Observed Pick-up counts
  data1$npick1 <- data1$NPICK
  data1$npick1 [data1$NPICK >= 3] <- 2
  data1$npick2 <- data1$npick1 - 1
  data1$obs_npick <- data1$npick1
  data1$npick2 <- factor(data1$npick2)
  

  
  
  ### Ordered probit utility computation for Pick-up
  #data1$const <- 1
  data1$cons <- coef_values[1]*data1$const
  expect_op <- data1[coef_names[1]] * coef_values[1]
  for (i in 2:(length(coef_names))) {
    expect_op <- expect_op + data1[coef_names[i]] * coef_values[i]
  }
  expect_op <- as.numeric(expect_op$const)
  
  
  ### Monte Carlo simulation for each age category
  indfor$NPICKUP1 <- count_veh(data1$const, expect_op, indfor$PICKUP1);
  indfor$NPICKUP2 <- count_veh(data1$const, expect_op, indfor$PICKUP2);
  indfor$NPICKUP3 <- count_veh(data1$const, expect_op, indfor$PICKUP3);
  
  
  data1$pred_npick <- indfor$NPICKUP1 + indfor$NPICKUP2 + indfor$NPICKUP3;
  data1$pred_npick[data1$pred_npick > 2] <- 2;
  comp_obs_pred <- list(data1$obs_npick, data1$pred_npick)
  
  ### Chart showing observed vs. predicted SUV counts
  multhist(comp_obs_pred, xlab="Number of Pick-ups in Household", ylab="Percent of Households", col = c("blue", "red"),  main = "Pick-up Count",
           legend.text = c("Observed", "Predicted"), breaks=seq(-0.5,3, by = 1), names.arg=c("0","1",">=2"), ylim=c(0,1),  freq = FALSE) 
  
  ### Final result of Vehicle Fleet Composition model 
  #write.table(indfor, file = paste(c(current_folder, output_file),collapse=''), row.names=FALSE, sep=",");
  
  return (indfor[c("NPICKUP1","NPICKUP2","NPICKUP3")])
  
}

